;**********************************************************************
;                                                                     *
;    Filename:							      *
;    Date:                                                  	      *
;    File Version:                                        	      *
;                                                                     *
;    Author:                                                          *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: "p16f876.inc"                                    *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: 1. Target:                     			      *
;                         					      *
;                                       			      *
;                 						      *
;                                                                     *
;**********************************************************************


	list      p=16f876            		; list directive to define processor
	#include <p16f876.inc>        		; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_ENABLE_ON & _LVP_OFF & _DEBUG_OFF & _CPD_OFF 

	
;***** LABEL DEFINITIONS	

;#define USE_UART				; define to enable UART
;#define USE_SSP				; define to enable SSP	

;***** ON-CHIP RAM ADDRESSES
RAM0		equ	0x20			; address of RAM bank 0
RAM1		equ	0xA0			; address of RAM bank 1
RAM2		equ	0x120			; address of RAM bank 2
RAM3		equ	0x1A0			; address of RAM bank 3
C_RAM		equ	0x70			; start of Common RAM

;***** ON-CHIP EEPROM ADDRESS
EEPROM		equ	0x2100			; start of on-chip EEPROM
	
;***** SYSTEM INITIALIZATION CONSTANTS

;***** SFR INIT CONSTANTS
#define	TRISA_INIT 	B'00000000'		; config. word for TRISA
#define TRISB_INIT 	B'00000000'	  	; config. word for TRISB
#define TRISC_INIT 	B'00000000'	  	; config. word for TRISC
#define PORTA_INIT 	B'00000000'	  	; config. word for PORTA
#define PORTB_INIT 	B'00000000'	  	; config. word for PORTB
#define PORTC_INIT 	B'00000000'	  	; config. word for PORTC
#define	INTCON_INIT	B'00000000'	  	; config. word for INTCON reg.
#define OPTION_INIT	B'00000000'	  	; config. word for OPTION_REG reg.
#define	ADCON0_INIT	B'00000000'	  	; config. word for ADCON0 reg.
#define	ADCON1_INIT	B'00000000'	  	; config. word for ADCON1 reg.
#define	CCP1CON_INIT	B'00000000'	  	; config. word for CCP1CON reg.
#define	CCP2CON_INIT	B'00000000'	  	; config. word for CCP2CON reg.
#define	T1CON_INIT	B'00000000'	  	; config. word for T1CON reg.
#define	T2CON_INIT	B'00000000'	  	; config. word for T2CON reg.
#define	PIE1_INIT	B'00000000'	  	; config. word for PIE1 reg.
#define	PIE2_INIT	B'00000000'	  	; config. word for PIE2 reg.

;***** USART PERIPHERAL CONSTANTS
#define TXSTA_INIT 	B'00000000'	  	; config. word for Tx status/ control reg.
#define RCSTA_INIT 	B'00000000'	  	; config. word for Rx status/ control reg.	
#define SPBRG_INIT	D'0'		  	; SPBRG value for xx baud-rate

;***** SSP PERIPHERAL CONSTANTS
#define SSPCON_INIT	B'00000000'		; config. word for SSPCON reg.
#define SSPCON2_INIT	B'00000000'		; config. word for SSPCON2 reg.
#define SSPSTAT_INIT	B'00000000'		; config. word for SSPSTAT reg.  	

;***** SYSTEM TIME / COUNTER CONSTANTS
		

;***** I/O REGISTER EQUATES AND DEFINITIONS
	

;***** END OF LABEL DEFINITIONS

;***** VARIABLE DEFINITIONS

	cblock	RAM0				; Bank 0 RAM addresses

	endc

	cblock	C_RAM				; common RAM addresses

w_temp                	  		  	; saved W reg. (for context saving) 
status_temp   		        	  	; saved STATUS reg. (for context saving)
pclath_temp					; saved PCLATH reg. (for context saving)
	
	endc


;***** END OF VARIABLE DEFINITIONS

;***** MACRO DEFINITIONS	

;***** END OF MACRO DEFINITIONS

;**********************************************************************
		ORG     0x000             	; processor reset vector
reset:		nop				; needed for ICD				
		clrf    PCLATH            	; ensure page bits are cleared
  		goto    start		  	; go to beginning of program


		ORG     0x004             	; interrupt vector location
isr:		movwf   w_temp            	; save off current W register contents
		swapf	STATUS,W          	; move STATUS register into W register		
		movwf	status_temp       	; save off contents of STATUS register
		movf	PCLATH,W		; move PCLATH register into W register
		movwf	pclath_temp		; save off contents of PCLATH register
		clrf	PCLATH			; select PAGE0

	; ISR code here...

isrx:		movf    pclath_temp,W     	; retrieve copy of PCLATH register
		movwf	PCLATH            	; restore pre-isr PCLATH register contents
		swapf   status_temp,W     	; retrieve copy of STATUS register
		movwf	STATUS            	; restore pre-isr STATUS register contents
		swapf   w_temp,F
		swapf   w_temp,W          	; restore pre-isr W register contents
		retfie                    	; return from interrupt

;***** BEGIN SYSTEM INITIALIZATION

start:
;***** THIS SECTION INITIALIZES THE SFR's
;***** BANK 0 registers		
		clrf	STATUS		  	; select BANK0
		movlw	PORTA_INIT		; get PORTA data config.
		movwf	PORTA			; write to PORTA 
		movlw	PORTB_INIT		; get PORTB data config.
		movwf	PORTB			; write to PORTB
		movlw	PORTC_INIT		; get PORTC data config.
		movwf	PORTC			; write to PORTC		
		movlw	ADCON0_INIT		; get ADCON0 config.
		movwf	ADCON0			; write to ADCON0
		movlw	T1CON_INIT		; get T1CON config.
		movwf	T1CON			; write to T1CON
		movlw	T2CON_INIT		; get T2CON config.
		movwf	T2CON			; write to T2CON
#ifdef USE_SSP
		movlw	SSPCON_INIT		; get SSPCON config.
		movwf	SSPCON			; write to SSPCON
		bcf	SSPCON,SSPEN		; disable SSP until fully configured
#endif
		movlw	CCP1CON_INIT		; get CCP1CON config.
		movwf	CCP1CON			; write to CCP1CON
		movlw	CCP2CON_INIT		; get CCP2CON config.
		movwf	CCP2CON			; write to CCP2CON
;***** BANK 1 registers
		bsf	STATUS,RP0		; select BANK1
		movlw	ADCON1_INIT		; get ADCON1 reg. config.
		movwf	ADCON1			; write to ADCON1 reg.
		movlw	TRISA_INIT		; get PORT A data direction config.
		movwf	TRISA			; write to PORT A TRIS Reg.
		movlw	TRISB_INIT		; get PORT B data direction config.
		movwf	TRISB			; write to PORT B TRIS Reg.
		movlw	TRISC_INIT		; get PORT C data direction config.
		movwf	TRISC			; write to PORT C TRIS Reg.
		movlw	OPTION_INIT		; get OPTION reg. config.
		movwf	OPTION_REG		; write to OPTION reg.
		movlw	PIE1_INIT		; get PIE1 config.
		movwf	PIE1			; write to PIE1
		movlw	PIE2_INIT		; get PIE2 config.
		movwf	PIE2			; write to PIE2
		movlw	INTCON_INIT		; get INTCON config.
		movwf	INTCON			; write to INTCON
#ifdef USE_SSP
		movlw	SSPCON2_INIT		; get SSPCON2 config.
		movwf	SSPCON2			; write to SSPCON2
		movlw	SSPSTAT_INIT		; get SSPSTAT config.
		movwf	SSPSTAT			; write to INTCON
#endif				
#ifdef USE_UART
;***** THIS SECTION CONFIGURES UART BAUD RATE (enter in BANK 1)
baud_set	movlw	SPBRG_INIT		; put baud-rate constant value in 'W' reg.					
		movwf	SPBRG			; write to baud-rate generator reg.		
;***** THIS SECTION CONFIGURES THE UART
		movlw	TXSTA_INIT		; get TXSTA config. word
		movwf	TXSTA			; write to TXSTA reg.	
		bcf	STATUS,RP0		; select BANK0
		movlw	RCSTA_INIT		; get RCSTA config. word
		movwf	RCSTA			; write to RCSTA reg.
;***** THIS SECTION ENABLES THE UART
uart_en		bsf	RCSTA,SPEN		; enable the port pins
		bsf	STATUS,RP0		; select BANK1
		bsf	TXSTA,TXEN		; enable Tx		
;***** UART is enabled
#endif
		clrf	STATUS			; select BANK0 		  
#ifdef USE_SSP
		clrf	SSPBUF			; clear SSP Tx buffer
		bsf	SSPCON,SSPEN		; enable SSP 
#endif
;***** END OF SYSTEM INITIALIZATION

;***** BEGINNING OF MAIN PROCESS

main:
	; Main application code here...

		ORG	EEPROM
	; Define EEPROM data here...

	END
